!{\src2tex{textfont=tt}}
!!****f* USPP2PAW/calc_shapef
!! NAME
!! calc_shapef
!!
!! FUNCTION
!! Compute normalized shape function and its moments (shpnrm)
!!
!! COPYRIGHT
!! Copyright (C) 1998-2014 ABINIT group (FJ, MT)
!! This file is distributed under the terms of the GNU General Public License
!! See http://www.gnu.org/copyleft/gpl.txt
!!
!! INPUTS
!!  option= choice governing the type of shape function: 0=from uspp, 1=(sinx/x)**2, 2=Bessel
!!  pawps
!!    %shpnrm(0)= Norm of compensation density
!!    %ushatden4pr2(wav_meshsz)= Compensation density (from USPP) multiplied by 4Pi.r2
!!  pawrad=datastructure containing mesh size definitions
!!  pshead
!!    %l_size= Max. value of l+1 leading to a non zero Gaunt coeffs
!!    %shape_type= Shape function type
!!    %wav_meshsz=Mesh size for partial waves and projectors inherited from uspp
!!  rcompfact= factor to be applied to rpaw to get rc_hat (rshape)
!!  un_log= Unit number for log file (comments)
!!
!! OUTPUT
!!  pawps
!!    %shapefunc(wav_meshsz)= Normalized shape function
!!    %shpnrm(l_size)= Normalisation of shape function for each l
!!  pshead
!!    %rc_hat= Radius for compensation charge
!!
!! PARENTS
!!      uspp2paw
!!
!! CHILDREN
!!      csimp,deducer0
!!
!! SOURCE

 subroutine calc_shapef(option,pawps,pawrad,pshead,rcompfact,un_log)

 use defs_basis
 use defs_pawps
 use m_csimp

 implicit none

!Arguments ------------------------------------
 integer :: option,un_log
 real(dp) :: rcompfact
 type(pawps_type)    :: pawps
 type(pawrad_type)   :: pawrad
 type(pshead_type)   :: pshead

!Local variables-------------------------------
 integer :: il,ir,meshsz
 real(dp) :: arg,dum1,dum2,intg,jbes1,jbes2,qr,shpnrm,shapesin,shapesins
 real(dp) :: al(2),ql(2)
 real(dp), allocatable :: ff(:)

!Statement functions -----------------------------------
 shapesin(arg) = (sin(pi*arg/pshead%rc_hat)/(pi*arg/pshead%rc_hat))**2
 shapesins(arg)= (1-(pi*arg/pshead%rc_hat)**2/6.d0*(1-(pi*arg/pshead%rc_hat)**2/20.d0))**2

! *************************************************************************

 meshsz=pshead%wav_meshsz

!Shape function read from uspp
!-----------------------------
 if (option==0) then
  pshead%rc_hat=pshead%rc_sph
  write(un_log,'(/,a)') '> USPP2PAW COMMENT:'
  write(un_log,'(a)')   '  Use shape function deduce from'
  write(un_log,'(a)')   '  compensation density computed in uspp.'
  pawps%shapefunc(2:meshsz)=pawps%ushatden4pr2(2:meshsz)/pawrad%rad(2:meshsz)**2/pawps%shpnrm(0)
  call deducer0(pawps%shapefunc,meshsz,pawrad)

!"Sin" shape function
!-----------------------------
 else if (option==1) then
  pshead%rc_hat=pshead%rc_sph
  if (rcompfact>=0.d0.and.rcompfact<=1.d0) pshead%rc_hat=rcompfact*pshead%rc_sph
  write(un_log,'(/,a)') '> USPP2PAW COMMENT:'
  write(un_log,'(a)')   '  Use "sinus" shape function with'
  write(un_log,'(a,f6.3)')   '  compensation charge radius=',pshead%rc_hat
  pawps%shapefunc(1)=1.d0
  do ir=2,meshsz
   if (pawrad%rad(ir)<=1.d-3) then
    pawps%shapefunc(ir)=shapesins(pawrad%rad(ir))
   else if (pawrad%rad(ir)<=pshead%rc_hat) then
    pawps%shapefunc(ir)=shapesin(pawrad%rad(ir))
   else
    pawps%shapefunc(ir)=zero
   endif
  enddo
  allocate(ff(meshsz));ff(1:meshsz)=pawps%shapefunc(1:meshsz)*pawrad%rad(1:meshsz)**2
  call csimp(ff,pawrad,meshsz,intg,r_for_intg=pshead%rc_hat);deallocate(ff)
  pawps%shapefunc(1:meshsz)=pawps%shapefunc(1:meshsz)/intg

!"Bessel" shape function
!-----------------------------
 else if (option==2) then
  pshead%rc_hat=pshead%rc_sph
  if (rcompfact>=0.d0.and.rcompfact<=1.d0) pshead%rc_hat=rcompfact*pshead%rc_sph
  write(un_log,'(/,a)') '> USPP2PAW COMMENT:'
  write(un_log,'(a)')   '  Use "Bessel" shape function with'
  write(un_log,'(a,f6.3)')   '  compensation charge radius=',pshead%rc_hat
  call shapebes(al,ql,0,pshead%rc_hat)
  do ir=1,meshsz
   if (pawrad%rad(ir)<=pshead%rc_hat) then
    qr=ql(1)*pawrad%rad(ir);CALL jbessel(jbes1,dum1,dum2,0,0,qr)
    qr=ql(2)*pawrad%rad(ir);CALL jbessel(jbes2,dum1,dum2,0,0,qr)
    pawps%shapefunc(ir)=al(1)*jbes1+al(2)*jbes2
   else
    pawps%shapefunc(ir)=zero
   endif
  enddo

!-----------------------------
 else
  write(un_log,'(/,a)') '> USPP2PAW ERROR (calc_shapef):'
  write(un_log,'(a)')   '  Invalid value of shape_type !'
  stop 'Program stopped before end'
 endif

!Compute moments of shape functions
!----------------------------------
 pawps%shpnrm(1)=one
 allocate(ff(meshsz))
 do il=2,pshead%l_size
  ff(1:meshsz)=pawps%shapefunc(1:meshsz)*pawrad%rad(1:meshsz)**(2*il)
  call csimp(ff,pawrad,meshsz,intg,r_for_intg=pshead%rc_hat)
  pawps%shpnrm(il)=1.d0/intg
 enddo
 deallocate(ff)

!Compute l-dependent shape functions
!-----------------------------------
 allocate(ff(meshsz))
 do il=1,pshead%l_size
  if (il==1) then
   ff(1:meshsz)=pawps%shapefunc(1:meshsz)
  else
   if (option==0.or.option==1) then
    do ir=1,meshsz
     ff(ir)=pawps%shapefunc(ir)*pawps%shpnrm(il)*pawrad%rad(ir)**(il-1)
    enddo
   else if (option==2) then
    call shapebes(al,ql,il-1,pshead%rc_hat)
    do ir=1,meshsz
     if (pawrad%rad(ir)<=pshead%rc_hat) then
      qr=ql(1)*pawrad%rad(ir);CALL jbessel(jbes1,dum1,dum2,il-1,0,qr)
      qr=ql(2)*pawrad%rad(ir);CALL jbessel(jbes2,dum1,dum2,il-1,0,qr)
      ff(ir)=al(1)*jbes1+al(2)*jbes2
     else
      ff(ir)=zero
     endif
    enddo
   else
    stop "BUG in calc_shapef !"
   endif
  endif
  pawps%shpf(:,il)=ff
 enddo
 deallocate(ff)

end subroutine
!!***
