 !{\src2tex{textfont=tt}}
!!****f* USPP2PAW/wrxmlps
!! NAME
!! wrxmlps
!!
!! FUNCTION
!! Write the paw data file in XML format
!!  See PAW-XML specification:
!!     https://wiki.fysik.dtu.dk/gpaw/setups/pawxml.html
!!
!! COPYRIGHT
!! Copyright (C) 2014-2018 ABINIT group (FJ, MT)
!! This file is distributed under the terms of the GNU General Public License
!! See http://www.gnu.org/copyleft/gpl.txt
!!
!! INPUT
!!  mesh_indx(:)= indexes the different meshes
!!  pshead
!!    %atomic_charge= Total atomic charge
!!    %basis_size= Number of elements for the paw nl basis
!!    %core_charge= Core charge
!!    %vlocopt=Option for the format of Vloc
!!              0: Vloc is Vbare (Blochl formulation)
!!              1: Vloc is VH(tnzc) (Kresse formulation)
!!              2: Vloc is VH(tnzc) (Kresse formulation without compensation in XC)
!!    %lambda= Lambda in gaussian type g(r)
!!    %lmn_size= Number of elements for the paw basis
!!    %lmn2_size=lmn_size*(lmn_size+1)/2
!!    %l_size = Maximum value of l+1 leading to a non zero Gaunt coef
!!    %mesh_size   = max. number of radial meshes
!!    %orbitals(basis_size)= Quantum number l for each basis function
!!    %pspxc_abinit= ABINIT code number for the exchange-correlation
!!    %pspversion= Version of the PAW format for Abinit
!!    %rc_hat= radius for shape function
!!    %rc_sph= Default PAW sphere radius
!!    %shape_type= Shape function type
!!    %sigma= Sigma for gaussian type g(r)
!!    %title= Title for pseudopotential
!!  pawps
!!   %coredens(:)  ! Gives the core density multiplied by 4Pi.r2
!!   %tcoredens(:) ! Gives the pseudized core density multiplied by 4Pi.r2
!!   %tvaledens(:) ! Gives the pseudized core density multiplied by 4Pi.r2 (up to r(vale_meshsz))
!!   %phi(:,:)        ! Gives, on the radial grid, the PAW atomic wavefunctions
!!   %tphi(:,:)       ! Gives, on the radial grid, the PAW atomic pseudowavefunctions
!!   %tproj(:,:)      ! Gives, on the radial grid, the PAW projectors for each basis function
!!   %dij0(:)         ! Part of the Dij term (non-local operator) frozen during SCF
!!   %shpf(:,:)       !shapefunction (mesh_size,l_size)
!!   %vhtnzc(:)       ! Gives the Hartree potential of the pseudo density
!!                    ! of the nucleus + core electrons of the atom
!!  radmesh(:)= datatypes of all the meshes
!!  xml_use_one_grid=option governing the number of radial grids in the XML dataset
!!
!! PARENTS
!!  wrpawps
!!
!! CHILDREN
!! 
!! SOURCE

 subroutine wrxmlps(fname,mesh_indx,pawps,pshead,radmesh,xml_use_one_grid)

 use defs_basis
 use defs_pawps

 implicit none

!Arguments ---------------------------------------------
!Scalars
 integer, parameter :: nmesh_max=6
 integer,intent(in) :: xml_use_one_grid
 character*(fnlen) :: fname
 type(pawps_type),intent(inout)    :: pawps
 type(pshead_type),intent(inout) :: pshead
!Arrays
 integer,intent(in) :: mesh_indx(nmesh_max)
 type(pawrad_type),intent(inout),target  :: radmesh(nmesh_max)

!Local variables ---------------------------------------
!Scalars
 integer, parameter :: xml_unit=14
 integer :: icoremesh,iprjmesh,ivalemesh,ivlocmesh
 integer :: ii,ib,il,iln,ilm,ilmn,ilmn0,ir,lmnmax,nbase,OK,imainmesh,iprojmesh,ishpfmesh
 integer :: jlmn,jlm,jln,j0lmn,klmn,nmesh,zz
 real(dp) :: radstp0,logstp0,sqr4pi,rc_hat
 character(len=4) :: flnm,flnm1
 character(len=20) :: nm,nm1
 character(len=5) ::f_type
 character(len=132) :: stripchar
 character(len=12) ::f_name
!Arrays
 integer :: mesh_indx_xml(nmesh_max),mesh_start(nmesh_max)
 real(dp),allocatable :: der(:),dum(:),ff(:),kij(:),kij0(:,:)
 character(len=3) :: gridt(nmesh_max)
 type(pawrad_type),pointer :: radmesh_xml(:)

! *************************************************************************

!Useful constants
 sqr4pi=sqrt(4*pi)
 nbase=pshead%basis_size
 zz=nint(pshead%atomic_charge)
 call find_atomname(zz,flnm)

!Quantum numbers for the partial wave basis
 lmnmax=0
 do il=1,pshead%basis_size
   lmnmax=lmnmax+2*pshead%orbitals(il)+1
 end do

!Header
 open(xml_unit,file=trim(fname)//'.xml',&
&          form='formatted')
 write(xml_unit,'("<?xml  version=""1.0""?>")')
 write(xml_unit,'("<paw_dataset version=""0.7"">")')
 write(xml_unit,'("<atom symbol=""",a,""" Z=""",f5.2,$)') trim(flnm),pshead%atomic_charge
 write(xml_unit,'(""" core=""", f5.2,""" valence=""",f5.2"""/>")')pshead%core_charge,&
&                                            (pshead%atomic_charge-pshead%core_charge)

!XC functional
 if(pshead%pspxc_abinit<0) then
   f_type="LIBXC"
   write(f_name,'(i6)') -pshead%pspxc_abinit
 end if
 select case(pshead%pspxc_abinit)
   case(2,-9001,-1009)
     f_type="LDA"
     f_name="PZ"
   case(4,-2001,-1002)
     f_type="LDA"
     f_name="W"
   case(5,-4001,-1004)
     f_type="LDA"
     f_name="HL"
   case(7,-12001,-1012)
     f_type="LDA"
     f_name="PW"
   case(11,-101130,-130101)
     f_type="GGA"
     f_name="PBE"
   case(14,-102130,-130102)
     f_type="GGA"
     f_name="revPBE"
   case(15,-117130,-130117)
     f_type="GGA"
     f_name="RPBE"
   case(-101116,-116101)
     f_type="GGA"
     f_name="PBEsol"
   case(-134109,-109134)
     f_type="GGA"
     f_name="PW91"
   case(-135120,-120135)
     f_type="GGA"
     f_name="AM05"
   case(-131106,-106131)
     f_type="GGA"
     f_name="BLYP"
   case default
     write(6, '(a)' ) '  Unable to identify XC functional!';stop
 end select
 write(xml_unit,'("<xc_functional type=""",a,""" name=""",a,"""/>")')TRIM(f_type),TRIM(f_name)
 write(xml_unit,'("<generator type=""translator"" name=""ABINIT2PAW"">")')
 write(xml_unit,'("</generator>")')

 write(xml_unit,'("<!--")')
 write(xml_unit,'("  ",a)') trim(pshead%title)
 write(xml_unit,'("  PAW functions generated on ",a,"/",a,"/",a)') trim(pshead%pspdat(5:6)),trim(pshead%pspdat(7:8)),&
&                 trim(pshead%pspdat(1:4))
 write(xml_unit,'("  -->")')
 write(xml_unit,'("<paw_radius rc=""",f17.14,"""/>")') pshead%rc_sph
 write(xml_unit,'("<valence_states>")')

!Valence states
 allocate(ff(nbase))
 ff=zero
 ilmn0=0
 do ib=1,nbase
   il=2*pshead%orbitals(ib)+1
   do ilmn=ilmn0+1,ilmn0+il
     ff(ib)=ff(ib)+pawps%rhoij0(ilmn*(ilmn+1)/2)
   end do
   ilmn0=ilmn0+il
 end do
 do ib=1,nbase
    call mkname(ib,flnm1)
    nm=stripchar('"'//flnm//flnm1//'"')
    write(xml_unit,'("  <state        l=""",i1,""" f=""",1pe14.7,$)')&
&        pshead%orbitals(ib),ff(ib)
    write(xml_unit,'(""" rc=""",f13.10,""" e=""",1pe14.7,""" id=",a6,"/>")')&
&        pshead%rc_sph,0.d0,TRIM(nm)
 enddo
 write(xml_unit,'("</valence_states>")')

!Radial meshes - Use several grids (from uspp) or only one
 if (xml_use_one_grid==0) then
   nmesh=pshead%nmesh
   mesh_indx_xml = mesh_indx
   radmesh_xml => radmesh
 else
!  "One grid" option is only available if all grids are of the same type
   if (any(radmesh(1:pshead%nmesh)%mesh_type/=radmesh(1)%mesh_type).or. &
&      any(abs(radmesh(1:pshead%nmesh)%rstep-radmesh(1)%rstep)>tol10).or. &
&      any(abs(radmesh(1:pshead%nmesh)%lstep-radmesh(1)%lstep)>tol10)) then
     stop "XML_ONE_GRID option is only available if all grids are of the same type!"
   end if
   nmesh=1
   mesh_indx_xml(:)=1
   allocate(radmesh_xml(1))
   imainmesh=1
   do ii=1,pshead%nmesh
     if (radmesh(ii)%mesh_size>radmesh(imainmesh)%mesh_size) imainmesh=ii
   end do
   radmesh_xml(1)%mesh_size =radmesh(imainmesh)%mesh_size
   radmesh_xml(1)%mesh_type =radmesh(imainmesh)%mesh_type
   radmesh_xml(1)%int_meshsz=radmesh(imainmesh)%int_meshsz
   radmesh_xml(1)%lstep     =radmesh(imainmesh)%lstep
   radmesh_xml(1)%rstep     =radmesh(imainmesh)%rstep
   radmesh_xml(1)%stepint   =radmesh(imainmesh)%stepint
   allocate(radmesh_xml(1)%rad    (radmesh_xml(1)%mesh_size))
   allocate(radmesh_xml(1)%radfact(radmesh_xml(1)%mesh_size))
   allocate(radmesh_xml(1)%simfact(radmesh_xml(1)%mesh_size))
   radmesh_xml(1)%rad   (:)=radmesh(imainmesh)%rad(:)
   radmesh_xml(1)%radfact(:)=radmesh(imainmesh)%radfact(:)
   radmesh_xml(1)%simfact(:)=radmesh(imainmesh)%simfact(:)
 end if

 do ii=1,nmesh
   allocate(der(radmesh_xml(ii)%mesh_size))
   select case(radmesh_xml(ii)%mesh_type)
     case(1)
       nm='r=d*i'
       gridt(ii)="lin"
       mesh_start(ii)=1
       radstp0=zero
       logstp0=radmesh_xml(ii)%rstep
       der=logstp0
     case(2)
       nm='r=a*(exp(d*i)-1)'
       gridt(ii)="log"
       mesh_start(ii)=1
       radstp0=radmesh_xml(ii)%rstep
       logstp0=radmesh_xml(ii)%lstep
       der(:)=logstp0*(radstp0+radmesh_xml(ii)%rad(:))
     case(3)
       nm='r=a*exp(d*i)'
       gridt(ii)="log"
       mesh_start(ii)=1;if (radmesh_xml(ii)%rad(1)<tol12) mesh_start(ii)=2
       radstp0=radmesh_xml(ii)%rstep
       logstp0=radmesh_xml(ii)%lstep
       der(:)=logstp0*radmesh_xml(ii)%rad(:)
     case default
       stop " Error in wrxmlps: mesh type not implemented!"
   end select
   write(xml_unit,'("<radial_grid eq=""",a,""" a=""",e23.16,$)') trim(nm),radstp0
   write(xml_unit,'(""" d=""",e23.16,""" istart=""0"" iend=""",i5,$)') logstp0,radmesh_xml(ii)%mesh_size-mesh_start(ii)
   write(xml_unit,'(""" id=""",a,i1,""">")') gridt(ii),ii
   write(xml_unit,'("  <values>")')
   write(xml_unit,'(3(1x,es23.16))') (radmesh_xml(ii)%rad(ir),ir=mesh_start(ii),radmesh_xml(ii)%mesh_size)
   write(xml_unit,'("  </values>")')
   write(xml_unit,'("  <derivatives>")')
   write(xml_unit,'(3(1x,es23.16))') (der(ir),ir=mesh_start(ii),radmesh_xml(ii)%mesh_size)
   write(xml_unit,'("  </derivatives>")')
   write(xml_unit,'("</radial_grid>")')
   deallocate(der)
 end do

!Compensation charge shape function
 rc_hat=pshead%rc_hat
 select case(pshead%shape_type)
   case(1)
     nm='exp'
     if(pshead%lambda==2) nm='gauss'
     if(nm=='exp') rc_hat=pshead%sigma
   case(2)
     nm='sinc'
   case(3)
     nm='bessel'
   case(-1)
     nm='num'
 end select
 if((pshead%shape_type==1).and.(pshead%lambda/=2)) then
   write(xml_unit,'("<shape_function type=""",a,""" rc=""",f19.16,""" lamb=""",i3,"""/>")')&
 &          trim(nm),rc_hat,pshead%lambda
 end if
 if((pshead%shape_type==2).or.(pshead%shape_type==3).or.(pshead%lambda==2)) then
   write(xml_unit,'("<shape_function type=""",a,""" rc=""",f19.16"""/>")') trim(nm),rc_hat
 end if
 if(pshead%shape_type==-1) then
   ishpfmesh=mesh_indx_xml(5)
   call mkname(ishpfmesh,flnm1)
   nm1=stripchar('"'//gridt(ishpfmesh)//flnm1//'"')
   do il=1,pshead%l_size
     write(xml_unit,'("<shape_function type=""",a,""" grid=",a," l=""",i1,""">")')&
 &          trim(nm),trim(nm1),il-1
     write(xml_unit,*) (pawps%shpf(ir,il),ir=mesh_start(ishpfmesh),radmesh_xml(ishpfmesh)%mesh_size)
     write(xml_unit,'("</shape_function>")')
   end do
 end if

!Core and pseudo-core densities
 icoremesh=mesh_indx_xml(3)
 write(xml_unit,'("<ae_core_density grid=""",a,i1,""" rc=""",f19.16,""">")') gridt(icoremesh),icoremesh,pshead%rc_cor
 write(xml_unit,'(3(1x,es23.16))') (sqr4pi*pawps%coredens(ir),ir=mesh_start(icoremesh),radmesh_xml(icoremesh)%mesh_size)
 write(xml_unit,'("</ae_core_density>")')
 write(xml_unit,'("<pseudo_core_density grid=""",a,i1,""" rc=""",f19.16,""">")') gridt(icoremesh),icoremesh,pshead%rc_cor
 write(xml_unit,'(3(1x,es23.16))') (sqr4pi*pawps%tcoredens(ir),ir=mesh_start(icoremesh),radmesh_xml(icoremesh)%mesh_size)
 write(xml_unit,'("</pseudo_core_density>")')

!Pseudo-valence densities
 if (pshead%pspversion>=4) then
   ivalemesh=mesh_indx_xml(6)
   write(xml_unit,'("<pseudo_valence_density grid=""",a,i1,""" rc=""",f19.16,""">")') gridt(ivalemesh),ivalemesh,pshead%rc_sph
   write(xml_unit,'(3(1x,es23.16))') (sqr4pi*pawps%tvaledens(ir),ir=mesh_start(ivalemesh),radmesh_xml(ivalemesh)%mesh_size)
   write(xml_unit,'("</pseudo_valence_density>")')
 end if

!Zero potential
 imainmesh=mesh_indx_xml(1)
 allocate(dum(radmesh_xml(imainmesh)%mesh_size),stat=OK)
 dum=0
 dum(1:pshead%sph_meshsz+5)=sqr4pi*pawps%vbare(1:pshead%sph_meshsz+5)
 write(xml_unit,'("<zero_potential grid=""",a,i1,""" rc=""",f19.16,""">")') gridt(imainmesh),imainmesh,pshead%rc_sph
 write(xml_unit,'(3(1x,es23.16))') (dum(ir),ir=mesh_start(imainmesh),radmesh_xml(imainmesh)%mesh_size)
 write(xml_unit,'("</zero_potential>")')
 deallocate(dum)

!Kresse-Joubert potential or Blochl local ionic potential
 ivlocmesh=mesh_indx_xml(4)
 if(pshead%vlocopt==1) then
   write(xml_unit,'("<kresse_joubert_local_ionic_potential grid=""",a,i1,""" rc=""",f19.16,""">")')&
&       gridt(ivlocmesh),ivlocmesh,pshead%rc_loc
   write(xml_unit,'(3(1x,es23.16))') (sqr4pi*pawps%vhtnzc(ir),ir=mesh_start(imainmesh),radmesh_xml(ivlocmesh)%mesh_size)
   write(xml_unit,'("</kresse_joubert_local_ionic_potential>")')
 end if
 if(pshead%vlocopt==2) then
   write(xml_unit,'("<blochl_local_ionic_potential grid=""",a,i1,""" rc=""",f19.16,""">")')&
&       gridt(ivlocmesh),ivlocmesh,pshead%rc_loc
   write(xml_unit,'(3(1x,es23.16))') (sqr4pi*pawps%vhtnzc(ir),ir=mesh_start(imainmesh),radmesh_xml(ivlocmesh)%mesh_size)
   write(xml_unit,'("</blochl_local_ionic_potential>")')
 endif

 do ib=1,nbase

!  AE and pseudo partial-waves
   allocate(dum(radmesh_xml(imainmesh)%mesh_size),stat=OK)
   call mkname(ib,flnm1)
   nm=stripchar('"'//flnm//flnm1//'"')
   dum=0
   dum(2:radmesh_xml(imainmesh)%mesh_size)=pawps%phi(2:radmesh_xml(imainmesh)%mesh_size,ib)&
&           /radmesh_xml(imainmesh)%rad(2:radmesh_xml(imainmesh)%mesh_size)
   call deducer0(dum,radmesh_xml(imainmesh)%mesh_size,radmesh_xml(imainmesh))
   write(xml_unit,'("<ae_partial_wave state=",a6," grid=""",a,i1,""">")')&
&          trim(nm),gridt(imainmesh),imainmesh
   write(xml_unit,'(3(1x,es23.16))') (dum(ii),ii=mesh_start(imainmesh),radmesh_xml(imainmesh)%mesh_size)
   write(xml_unit,'("</ae_partial_wave>")')
   dum=0
   dum(2:radmesh_xml(imainmesh)%mesh_size)=pawps%tphi(2:radmesh_xml(imainmesh)%mesh_size,ib)&
&           /radmesh_xml(imainmesh)%rad(2:radmesh_xml(imainmesh)%mesh_size)
   call deducer0(dum,radmesh_xml(imainmesh)%mesh_size,radmesh_xml(imainmesh))
   write(xml_unit,'("<pseudo_partial_wave state=",a6," grid=""",a,i1,""">")')&
&          trim(nm),gridt(imainmesh),imainmesh
   write(xml_unit,'(3(1x,es23.16))') (dum(ii),ii=mesh_start(imainmesh),radmesh_xml(imainmesh)%mesh_size)
   write(xml_unit,'("</pseudo_partial_wave>")')
   deallocate(dum)

!  NL projectors
   iprojmesh=mesh_indx_xml(2)
   allocate(dum(radmesh_xml(iprojmesh)%mesh_size+5),stat=OK)
   dum=0
   dum(2:radmesh_xml(iprojmesh)%mesh_size+5)=pawps%tproj(2:radmesh_xml(iprojmesh)%mesh_size+5,ib)&
&           /radmesh_xml(iprojmesh)%rad(2:radmesh_xml(iprojmesh)%mesh_size+5)
   call deducer0(dum,radmesh_xml(iprojmesh)%mesh_size,radmesh_xml(iprojmesh))
   write(xml_unit,'("<projector_function state=",a6," grid=""",a,i1,""">")') &
&            trim(nm),gridt(iprojmesh),iprojmesh
   write(xml_unit,'(3(1x,es23.16))') (dum(ii),ii=mesh_start(iprojmesh),radmesh_xml(iprojmesh)%mesh_size+5)
   write(xml_unit,'("</projector_function>")')
   deallocate (dum)

 end do

!Kinetic energy differences
 allocate (kij(pshead%lmn2_size))
 kij=pawps%kij
 allocate (kij0(pshead%basis_size,pshead%basis_size))
 kij0=zero
 do jlmn=1,pshead%lmn_size
   j0lmn=jlmn*(jlmn-1)/2
   jlm=pshead%indlmn(4,jlmn);jln=pshead%indlmn(5,jlmn)
   do ilmn=1,jlmn
     klmn=j0lmn+ilmn
     ilm=pshead%indlmn(4,ilmn);iln=pshead%indlmn(5,ilmn)
     if (ilm==jlm) then
       kij0(iln,jln)=kij(klmn)
       kij0(jln,iln)=kij(klmn)
     end if
   end do
 end do
 write(xml_unit,'("<kinetic_energy_differences>")')
 write(xml_unit,'(3(1x,es23.16))') ((kij0(ir,ii),ii=1,pshead%basis_size),ir=1,pshead%basis_size)
 write(xml_unit,'("</kinetic_energy_differences>")')
 write(xml_unit,'("</paw_dataset>")')
 close(xml_unit)
 deallocate (kij,kij0)

 if (xml_use_one_grid/=0) then
   deallocate(radmesh_xml(1)%rad)
   deallocate(radmesh_xml(1)%radfact)
   deallocate(radmesh_xml(1)%simfact)
   deallocate(radmesh_xml)
 end if

 end subroutine wrxmlps
!!***

! *************************************************************************
! Additional routines and functions
! *************************************************************************

 subroutine find_atomname(zz,flnm)
!Return atom (short) name from atom number
 use defs_basis
 implicit none
 integer,intent(in) :: zz
 character (len=4), intent(out) :: flnm
 select case(zz)
   case(1)
     flnm='H'
   case(2)
     flnm='He'
   case(3)
     flnm='Li'
   case(4)
     flnm='Be'
   case(5)
     flnm='B'
   case(6)
     flnm='C'
   case(7)
     flnm='N'
   case(8)
     flnm='O'
   case(9)
     flnm='F'
   case(10)
     flnm='Ne'
   case(11)
     flnm='Na'
   case(12)
     flnm='Mg'
   case(13)
     flnm='Al'
   case(14)
     flnm='Si'
   case(15)
     flnm='P'
   case(16)
     flnm='S'
   case(17)
     flnm='Cl'
   case(18)
     flnm='Ar'
   case(19)
     flnm='K'
   case(20)
     flnm='Ca'
   case(21)
     flnm='Sc'
   case(22)
     flnm='Ti'
   case(23)
     flnm='V'
   case(24)
     flnm='Cr'
   case(25)
     flnm='Mn'
   case(26)
     flnm='Fe'
   case(27)
     flnm='Co'
   case(28)
     flnm='Ni'
   case(29)
     flnm='Cu'
   case(30)
     flnm='Zn'
   case(31)
     flnm='Ga'
   case(32)
     flnm='Ge'
   case(33)
     flnm='As'
   case(34)
     flnm='Se'
   case(35)
     flnm='Br'
   case(36)
     flnm='Kr'
   case(37)
     flnm='Rb'
   case(38)
     flnm='Sr'
   case(39)
     flnm='Y'
   case(40)
     flnm='Zr'
   case(41)
     flnm='Nb'
   case(42)
     flnm='Mo'
   case(43)
     flnm='Tc'
   case(44)
     flnm='Ru'
   case(45)
     flnm='Rh'
   case(46)
     flnm='Pd'
   case(47)
     flnm='Ag'
   case(48)
     flnm='Cd'
   case(49)
     flnm='In'
   case(50)
     flnm='Sn'
   case(51)
     flnm='Sb'
   case(52)
     flnm='Te'
   case(53)
     flnm='I'
   case(54)
     flnm='Xe'
   case(55)
     flnm='Cs'
   case(56)
     flnm='Ba'
   case(57)
     flnm='La'
   case(58)
     flnm='Ce'
   case(59)
     flnm='Pr'
   case(60)
     flnm='Nd'
   case(61)
     flnm='Pm'
   case(62)
     flnm='Sm'
   case(63)
     flnm='Eu'
   case(64)
     flnm='Gd'
   case(65)
     flnm='Tb'
   case(66)
     flnm='Dy'
   case(67)
     flnm='Ho'
   case(68)
     flnm='Er'
   case(69)
     flnm='Tm'
   case(70)
     flnm='Yb'
   case(71)
     flnm='Lu'
   case(72)
     flnm='Hf'
   case(73)
     flnm='Ta'
   case(74)
     flnm='W'
   case(75)
     flnm='Re'
   case(76)
     flnm='Os'
   case(77)
     flnm='Ir'
   case(78)
     flnm='Pt'
   case(79)
     flnm='Au'
   case(80)
     flnm='Hg'
   case(81)
     flnm='Tl'
   case(82)
     flnm='Pb'
   case(83)
     flnm='Bi'
   case(84)
     flnm='Po'
   case(85)
     flnm='At'
   case(86)
     flnm='Rn'
   case(87)
     flnm='Fr'
   case(88)
     flnm='Ra'
   case(89)
     flnm='Ac'
   case(90)
     flnm='Th'
   case(91)
     flnm='Pa'
   case(92)
     flnm='U'
   case(93)
     flnm='Np'
   case(94)
     flnm='Pu'
   case(95)
     flnm='Am'
   case(96)
     flnm='Cm'
   case(97)
     flnm='Bk'
   case(98)
     flnm='Cf'
   case(99)
     flnm='Es'
   case(100)
     flnm='Fm'
   case(101)
     flnm='Md'
   case(102)
     flnm='No'
   case(103)
     flnm='Lw'
 end select
 end subroutine find_atomname

! *************************************************************************

 subroutine mkname(ii,stuff)
!Take an integer (<4 digits) and return it
!in the form of a character string
 CHARACTER(4) stuff
 INTEGER ii,i1,i10,i100,i1000
 stuff='?'
 IF (ii.GT.10000) ii=MOD(ii,10000)
 i1000=ii/1000
 i100=(ii-1000*i1000)/100
 i10=(ii-1000*i1000-100*i100)/10
 i1=(ii-1000*i1000-100*i100-10*i10)
 IF (ii>=1000) THEN
    stuff=CHAR(i1000+48)//CHAR(i100+48)//CHAR(i10+48)//CHAR(i1+48)
    RETURN
 ENDIF
 IF (ii>=100) THEN
    stuff=CHAR(i100+48)//CHAR(i10+48)//CHAR(i1+48)
    RETURN
 ENDIF
 IF (ii>=10) THEN
    stuff=CHAR(i10+48)//CHAR(i1+48)
    RETURN
 ENDIF
 IF (ii>=0) stuff=CHAR(i1+48)
 RETURN
 end subroutine mkname

! *************************************************************************

 function stripchar(inputchar)
!Strip a character string
 CHARACTER(132) :: stripchar
 CHARACTER*(*), INTENT(IN) :: inputchar
 INTEGER :: ii,jj,nn
 nn=LEN(inputchar)
 do ii=1,132
   stripchar(ii:ii)=''
 enddo
 jj=0
 do ii=1,nn
   if (inputchar(ii:ii) /= '') then
      jj=jj+1
      stripchar(jj:jj)=inputchar(ii:ii)
   endif
 enddo
 end function stripchar

! *************************************************************************
